//
//  ViewController.swift
//  PowerGuard
//
//  Created by Phidgets on 2018-02-01.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = PowerGuard()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var powerEnabledSwitch: UISwitch!
    @IBOutlet var overVoltageSlider: UISlider!
    @IBOutlet var overVoltageLabel: UILabel!
    @IBOutlet var fanModeSegment: UISegmentedControl!
    @IBOutlet var stackView: UIStackView!
    @IBOutlet var errorStack: UIStackView!
    @IBOutlet var errorText: UITextView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! PowerGuard
            
            let minOverVoltage = try Float(attachedDevice.getMinOverVoltage())
            let maxOverVoltage = try Float(attachedDevice.getMaxOverVoltage())
            
            let fanMode = try attachedDevice.getFanMode()
            let powerEnabled = try attachedDevice.getPowerEnabled()
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.overVoltageSlider.minimumValue = minOverVoltage
                self.overVoltageSlider.maximumValue = maxOverVoltage
                self.overVoltageSlider.value = minOverVoltage
                
                self.powerEnabledSwitch.isOn = powerEnabled
                self.fanModeSegment.selectedSegmentIndex = (fanMode == FanMode.auto) ? 0 : (fanMode == FanMode.on) ? 1 : 2
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
                self.errorStack.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }

    func outputErrorMessage(errorTitle:String, errorMessage:String){
        DispatchQueue.main.async {
            self.errorStack.isHidden = false
            self.errorText.text = self.errorText.text + errorTitle + "\n" + errorMessage + "\n\n"
        }
    }
    
    //MARK: GUI controls
    @IBAction func setOverVoltage(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setOverVoltage(Double(slider.value))
            DispatchQueue.main.async {
                self.overVoltageLabel.text = String(format:"%.2f V", slider.value)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setPowerEnabled(_ sender: Any) {
        let swtch = sender as! UISwitch
        do{
            try ch.setPowerEnabled(swtch.isOn)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setFanMode(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        do{
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setFanMode(FanMode.auto)
                break
            case 1:
                try ch.setFanMode(FanMode.on)
                break
            case 2:
                try ch.setFanMode(FanMode.off)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func clearErrors(_ sender: Any) {
        DispatchQueue.main.async {
            self.errorText.text = ""
        }
    }
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
}

